VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "STDlg"
Attribute VB_GlobalNameSpace = True
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
Option Explicit
'=====
'STDlg
'=====
'
'
'OVERVIEW
'========

'Simple TaskDialog support for VB6.
'
'TaskDialogs are an enhanced "message box" available beginning with
'Window Vista.  VB6 has no direct way to raise these dialogs.
'
'This class is designed to offer limited access to THE SIMPLE FORM of
'TaskDialog with a fallback to VB6's MsgBox if the program is run
'with an older version of comctl32.dll.
'
'The mapping is not perfect:
'
'   o MsgBox() can support additional options for buttons, icons, etc.
'     so this code only supports the subset TaskDialog() supports.
'
'   o TaskDialog also supports a Close button and result that has no
'     MsgBox equivalent.
'
'   o TaskDialog supports ad hoc combinations of buttons.  Where the
'     combination is not supported by VB6's MsgBox() only an Ok
'     button will be used.
'
'     For this reason if you want to have these calls degrade
'     gracefully on downlevel systems be sure to use mutually-
'     supported combinations.
'
'   o A TaskDialog custom icon will result in no icon when MsgBox is
'     invoked (no CC6 manifest or a downlevel system such as WinXP
'     or earlier).
'
'   o This version incorporates InitCommonControls() as a secondary
'     method you may find useful.
'
'If you call STDlg.TaskDialog() with a button combination that MsgBox()
'does not support you will produce an "Ok only" MsgBox dialog.
'
'
'WHY USE STDLG.TASKDIALOG()?
'===========================
'
'More advanced TaskDialogs require callback routines, advanced
'data structures, and a call to TaskDialogIndirect() instead of just
'the simple TaskDialog() calls we do here.
'
'So what do you get?
'
'Three things really:
'
'   o A newer look and feel on a newer OS (Vista or later) than the
'     traditional msgBox() or MessageBox() API call.
'
'   o The MainInstruction string, which appears in a larger and
'     different colored font than the Content text.
'
'   o The ability to use a custom icon.
'
'So I agree, not all that much really.
'
'
'NOTES
'=====
'
'   o In order to select Common Controls 6.1 (or later) you need to
'     use an application manifest selecting "Common Controls 6" and be
'     running on Vista or later.
'
'   o This class is defined such that you get a global default instance
'     by default.  You do not need to create an instance of STDlg in
'     your program.
'
'   o A custom icon should be stored as a program "Icon" resource, and
'     given a numeric ID value.  Specify this ID in the VB6 IDE
'     Resource Editor add-in with a leading #-sign, as in:
'
'                                #101
'
'     You pass this as the CustomIconID parameter as an Integer value
'     (in this example 101).
'
'     Make sure your Icon resource contains a 32x32 pixel image,
'     preferably 256 colors but 16 color icon images will work fine.
'
'
'PARAMETERS
'==========
'
'hWndParentForm As Long
'       hWnd of parent Form that should "own" the dialog or
'       0 for none.
'
'Content As String
'       The message body to be displayed (similar to the MsgBox Prompt
'       parameter).
'
'Optional Buttons As TASKDIALOG_COMMON_BUTTON_FLAGS
'       A bit mask of buttons to display.  Default value or 0 results
'       in just an Ok button.
'
'Optional Icon As TASKDIALOG_STANDARD_ICONS
'       The standard icon (or no icon, or a Custom indicator which
'       has the same value as "no icon").
'
'Optional CustomIconID As Integer
'       A custom icon's resource ID if any.
'
'Optional MainInstruction As String
'       The larger-font "title" to be displayed.  If omitted or an
'       empty String value is passed then App.EXEName is used.
'
'Optional WindowTitle As String
'       The dialog window caption.  If omitted or an empty String
'       then App.EXEName is used.  Same as a MsgBox() Title parameter.
'

Private Const S_OK = &H0&

Public Enum TASKDIALOG_COMMON_BUTTON_FLAGS
    TDCBF_OK_BUTTON = 1
    TDCBF_YES_BUTTON = 2
    TDCBF_NO_BUTTON = 4
    TDCBF_CANCEL_BUTTON = 8
    TDCBF_RETRY_BUTTON = &H10&
    TDCBF_CLOSE_BUTTON = &H20&
End Enum

Public Enum TASKDIALOG_STANDARD_ICONS
    TD_NO_ICON = 0
    TD_CUSTOM_ICON = 0
    TD_WARNING_ICON = -1
    TD_ERROR_ICON = -2
    TD_INFORMATION_ICON = -3
    TD_SHIELD_ICON = -4
End Enum

Public Enum TASKDIALOG_RESULTS
    IDOK = 1
    IDCANCEL = 2
    IDRETRY = 4
    IDYES = 6
    IDNO = 7
    IDCLOSE = 8
End Enum

Private Type DLLVERSIONINFO
    cbSize As Long
    dwMajor As Long
    dwMinor As Long
    dwBuildNumber As Long
    dwPlatformID As Long
End Type

Private Declare Function DllGetVersion Lib "comctl32" (ByVal pdvi As Long) As Long
Private Declare Function FreeLibrary Lib "kernel32" (ByVal hLibModule As Long) As Long
Private Declare Function GetProcAddress Lib "kernel32" ( _
    ByVal hModule As Long, _
    ByVal lpProcName As String) As Long
Private Declare Function LoadLibrary Lib "kernel32" Alias "LoadLibraryW" ( _
    ByVal lpLibFileName As Long) As Long
Private Declare Function TaskDialogAPI Lib "comctl32" Alias "TaskDialog" ( _
    ByVal hWndParent As Long, _
    ByVal hInstance As Long, _
    ByVal pszWindowTitle As Long, _
    ByVal pszMainInstruction As Long, _
    ByVal pszContent As Long, _
    ByVal dwCommonButtons As TASKDIALOG_COMMON_BUTTON_FLAGS, _
    ByVal pszIcon As Long, _
    ByRef pnButton As Long) As Long

Private mCCVersion As Single

Public Function TaskDialog( _
    ByVal hWndParentForm As Long, _
    ByVal Content As String, _
    Optional ByVal Buttons As TASKDIALOG_COMMON_BUTTON_FLAGS = TDCBF_OK_BUTTON, _
    Optional ByVal Icon As TASKDIALOG_STANDARD_ICONS = 0, _
    Optional ByVal CustomIconID As Integer = 0, _
    Optional ByVal MainInstruction As String = "", _
    Optional ByVal WindowTitle As String = "") As TASKDIALOG_RESULTS
    Dim HResult As Long
    
    If mCCVersion > 6# Then
        If Len(MainInstruction) < 1 Then MainInstruction = App.EXEName
        If Len(WindowTitle) < 1 Then WindowTitle = App.EXEName
        If Icon = TD_NO_ICON Then Icon = CustomIconID
        HResult = TaskDialogAPI(hWndParentForm, _
                                App.hInstance, _
                                StrPtr(WindowTitle), _
                                StrPtr(MainInstruction), _
                                StrPtr(Content), _
                                Buttons, _
                                Icon And &HFFFF&, _
                                TaskDialog)
        If HResult <> S_OK Then
            Err.Raise &H800446FF, "STDlg.TaskDialog", _
                      "System error " & CStr(HResult) & " (" & Hex$(HResult) & ")"
        End If
    Else
        Select Case Buttons
            Case TDCBF_OK_BUTTON
                Buttons = vbOKOnly
            Case TDCBF_OK_BUTTON Or TDCBF_CANCEL_BUTTON
                Buttons = vbOKCancel
            Case TDCBF_YES_BUTTON Or TDCBF_NO_BUTTON Or TDCBF_CANCEL_BUTTON
                Buttons = vbYesNoCancel
            Case TDCBF_YES_BUTTON Or TDCBF_NO_BUTTON
                Buttons = vbYesNo
            Case TDCBF_RETRY_BUTTON Or TDCBF_CANCEL_BUTTON
                Buttons = vbRetryCancel
            Case Else
                Buttons = vbOKOnly
        End Select
        Select Case Icon
            Case TD_WARNING_ICON
                Buttons = Buttons Or vbExclamation
            Case TD_ERROR_ICON
                Buttons = Buttons Or vbCritical
            Case TD_INFORMATION_ICON
                Buttons = Buttons Or vbInformation
        End Select
        If Len(MainInstruction) > 0 Then
            Content = MainInstruction & vbNewLine & vbNewLine & Content
        End If
        If Len(WindowTitle) > 0 Then
            TaskDialog = MsgBox(Content, Buttons, WindowTitle)
        Else
            TaskDialog = MsgBox(Content, Buttons)
        End If
        Select Case TaskDialog
            Case vbOK
                TaskDialog = IDOK
            Case vbCancel
                TaskDialog = IDCANCEL
            Case vbRetry
                TaskDialog = IDRETRY
            Case vbYes
                TaskDialog = IDYES
            Case vbNo
                TaskDialog = IDNO
        End Select
    End If
End Function

Private Function GetComCtlVersion() As Single
    Dim hMod As Long
    Dim tDVI As DLLVERSIONINFO

    hMod = LoadLibrary(StrPtr("comctl32.dll"))
    If hMod <> 0 Then
        If GetProcAddress(hMod, "DllGetVersion") <> 0 Then
            tDVI.cbSize = Len(tDVI)
            If DllGetVersion(VarPtr(tDVI)) = S_OK Then
                GetComCtlVersion = tDVI.dwMajor + tDVI.dwMinor * 0.0001
            End If
        Else
            'If GetProcAddress failed, then the DLL is a version previous to the one
            'shipped with IE 3.x.
            GetComCtlVersion = 4
        End If
        FreeLibrary hMod
    End If
End Function

Private Sub Class_Initialize()
    mCCVersion = GetComCtlVersion()
End Sub
